@@ -3,6 +3,8 @@ package FFI::Build;
 use strict;
 use warnings;
 use 5.008004;
+use FFI::Build::Plugin;
+use FFI::Build::PluginData qw( plugin_data );
 use FFI::Build::File::Library;
 use Carp ();
 use File::Glob ();
@@ -12,7 +14,7 @@ use Capture::Tiny ();
 use File::Path ();
 
 # ABSTRACT: Build shared libraries for use with FFI
-our $VERSION = '1.60_01'; # TRIAL VERSION
+our $VERSION = '1.61_01'; # TRIAL VERSION
 
 # Platypus-Man,
 # Platypus-Man,
@@ -24,6 +26,22 @@ our $VERSION = '1.60_01'; # TRIAL VERSION
 # There Goes The Platypus-Man
 
 
+{
+  my $plugins = FFI::Build::Plugin->new;
+  # PLUGIN: require
+  # ARGS: NONE
+  $plugins->call('build-require');
+  sub _plugins { $plugins ||= FFI::Build::Plugin->new };
+}
+
+sub import
+{
+  my @caller = caller;
+  # PLUGIN: import
+  # ARGS: @caller, \@args
+  _plugins->call('build-import', \@caller, \@_);
+}
+
 sub _native_name
 {
   my($self, $name) = @_;
@@ -36,6 +54,10 @@ sub new
 
   Carp::croak "name is required" unless defined $name;
 
+  # PLUGIN: new-pre
+  # ARGS: $name, \%args
+  _plugins->call('build-new-pre', $name, \%args);
+
   my $self = bless {
     source   => [],
     cflags_I => [],
@@ -86,6 +108,10 @@ sub new
 
   $self->source(ref $args{source} ? @{ $args{source} } : ($args{source})) if $args{source};
 
+  # PLUGIN: new-post
+  # ARGS: $self
+  _plugins->call('build-new-post', $self);
+
   $self;
 }
 
@@ -204,12 +230,20 @@ sub build
 {
   my($self) = @_;
 
+  # PLUGIN: build
+  # ARGS: $self
+  _plugins->call('build-build', $self);
+
   my @objects;
 
   my $ld = $self->platform->ld;
 
   foreach my $source ($self->source)
   {
+    # PLUGIN: build-item
+    # ARGS: $self, $source
+    _plugins->call('build-build-item', $self, $source);
+
     if($source->can('build_all'))
     {
       my $count = scalar $self->source;
@@ -259,6 +293,10 @@ sub build
     $self->platform->flag_library_output($self->file->path),
   );
 
+  # PLUGIN: build-link
+  # ARGS: $self, \@cmd
+  _plugins->call('build-build-link', $self, \@cmd);
+
   my($out, $exit) = Capture::Tiny::capture_merged(sub {
     $self->platform->run(@cmd);
   });
@@ -277,6 +315,10 @@ sub build
     print "LD @{[ $self->file->path ]}\n";
   }
 
+  # PLUGIN: link-postlink
+  # ARGS: $self, \@cmd
+  _plugins->call('build-build-postlink', $self);
+
   $self->file;
 }
 
@@ -285,13 +327,24 @@ sub clean
 {
   my($self) = @_;
   my $dll = $self->file->path;
-  unlink $dll if -f $dll;
+  if(-f $dll)
+  {
+    # PLUGIN: clean
+    # ARGS: $self, $path
+    _plugins->call('build-clean', $self, $dll);
+    unlink $dll;
+  }
   foreach my $source ($self->source)
   {
     my $dir = File::Spec->catdir($source->dirname, $self->buildname);
     if(-d $dir)
     {
-      unlink $_ for File::Glob::bsd_glob("$dir/*");
+      foreach my $path (File::Glob::bsd_glob("$dir/*"))
+      {
+        _plugins->call('build-clean', $self, $path);
+        unlink $path;
+      }
+      _plugins->call('build-clean', $self, $dir);
       rmdir $dir;
     }
   }
@@ -311,7 +364,7 @@ FFI::Build - Build shared libraries for use with FFI
 
 =head1 VERSION
 
-version 1.60_01
+version 1.61_01
 
 =head1 SYNOPSIS
 
